"""Const for Tests."""

STABLE_VERSION = "2020.99.9"
BETA_VERSION = "2020.99.9b0"
DEV_VERSION = "2020.99.0.dev19700101"

STABLE_VERSION_BETA_WEEK = "2020.98.9"
BETA_VERSION_BETA_WEEK = "2020.99.9b0"

HEADERS = {"Content-Type": "application/json"}
